@@ -8,11 +8,14 @@
 #' with values not present in the original data (like a value 0.3334 in a 0-1 coded variable). Secondly, predictive mean
 #' matching tries to raise the variance in the resulting conditional distributions to a realistic level and, as such, 
 #' allows to do multiple imputation when repeating the call to missRanger(). The iterative chaining stops as soon as \code{maxiter}
-#' is reached or if the average out-of-bag estimate of performance stops improving.
+#' is reached or if the average out-of-bag estimate of performance stops improving. In the latter case, except for the first iteration,
+#' the second last (i.e. best) imputed data is returned.
 #' @param data A \code{data.frame} with missing values to impute.
 #' @param maxiter Maximum number of chaining iterations.
 #' @param pmm.k Number of candidate non-missing values to sample from in the predictive mean matching step. 0 to avoid this step.
 #' @param seed Integer seed to initialize the random generator.
+#' @param verbose Controls how much info is printed to screen. 0 to print nothing. 1 (default) to print a "." per iteration and 
+#'                variable, 2 to print the OOB prediction error per iteration and variable (1 minus R-squared for regression).
 #' @param ... Arguments passed to \code{ranger}. If the data set is large, better use less trees 
 #' (e.g. \code{num.trees = 100}) and/or a low value of \code{sample.fraction}. 
 #' The following arguments are incompatible: \code{formula}, \code{data}, \code{write.forest}, 
@@ -36,9 +39,10 @@
 #' # With extra trees algorithm
 #' irisImputed_et <- missRanger(irisWithNA, pmm.k = 3, num.trees = 100, splitrule = "extratrees")
 #' head(irisImputed_et)
-missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, ...) {
-  
-  cat("\nMissing value imputation by chained tree ensembles")
+missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, verbose = 1, ...) {
+  if (verbose > 0) {
+    cat("\nMissing value imputation by chained tree ensembles\n")
+  }
   
   stopifnot(is.data.frame(data), dim(data) >= 1L, 
             is.numeric(maxiter), length(maxiter) == 1L, maxiter >= 1L,
@@ -51,10 +55,9 @@ missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, ...) {
     set.seed(seed)
   }  
   
-  allVars <- names(which(vapply(data, function(z) (is.factor(z) || is.numeric(z)) && any(!is.na(z)), 
-                                TRUE)))
+  allVars <- names(which(vapply(data, function(z) (is.factor(z) || is.numeric(z)) && any(!is.na(z)), TRUE)))
   
-  if (length(allVars) < ncol(data)) {
+  if (verbose > 0 && length(allVars) < ncol(data)) {
     cat("\n  Variables ignored in imputation (wrong data type or all values missing: ")
     cat(setdiff(names(data), allVars), sep = ", ")
   }
@@ -68,14 +71,20 @@ missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, ...) {
     return(data)
   }
   
-  j <- 1L
+  verboseDigits <- 4  # prediction of OOB prediction errors (if verbose = 2)
+  j <- 1L             # iterator
   predError <- rep(1, length(visit.seq))
   names(predError) <- visit.seq
-  crit <- TRUE
+  crit <- TRUE        # criterion on OOB prediction error to keep iterating
   completed <- setdiff(allVars, visit.seq)
   
+  if (verbose >= 2) {
+    cat("\n", abbreviate(visit.seq, minlength = verboseDigits + 2), sep = "\t")
+  }
   while (crit && j <= maxiter) {
-    cat("\n  missRanger iteration ", j, ":", sep = "")
+    if (verbose > 0) {
+      cat("\niter ", j, ":\t", sep = "")
+    }
     data.last <- data
     predErrorLast <- predError
     
@@ -101,14 +110,20 @@ missRanger <- function(data, maxiter = 10L, pmm.k = 0L, seed = NULL, ...) {
       }
       
       completed <- union(completed, v)
-      cat(".")
+      if (verbose == 1) {
+        cat(".")
+      } else if (verbose >= 2) {
+        cat(format(round(predError[[v]], verboseDigits), nsmall = verboseDigits), "\t")
+
+      }
     }
-    
-    cat("done")
+
     j <- j + 1L
     crit <- mean(predError) < mean(predErrorLast)
   }
   
-  cat("\n")
+  if (verbose > 0) {
+    cat("\n")
+  }
   if (j == 2L || (j == maxiter && crit)) data else data.last
 }
